@@ -7,7 +7,7 @@ use Math::Factor qw/factor match/;
 
 my (@numbers, %factors, %matches, $format_factors,
     $format_match_number, $format_match_matches);
-
+    
 @numbers = qw(9 30107);
 
 %factors = factor(\@numbers);
@@ -26,7 +26,13 @@ croak $@ if $@;
 
 foreach (sort {$a <=> $b} keys %factors) {
     $ul = '-' x length;
-    write;
+    write; local $, = "\t";
+    
+    print <<"EOT";
+@{$factors{$_}}
+
+
+EOT
 }	
 			
 print <<'EOT';
@@ -63,10 +69,6 @@ $format_factors = '
 $_
 @<<<<<<<<<<<<<<<<<<<<<<<<<
 $ul
-@<<<<<<<<<<<<<<<<<<<<<<<<<
-"@{$factors{$_}}"
-
-
 .
 ';
 
