@@ -1,9 +1,9 @@
 #
-# $Id: Factor.pm,v 0.08 2004/01/10 23:07:03 sts Exp $
+# $Id: Factor.pm,v 0.09 2004/01/12 23:59:45 sts Exp $
 
 package Math::Factor;
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 use integer;
 use strict 'vars';
@@ -25,6 +25,8 @@ our (@EXPORT_OK, %EXPORT_TAGS, @subs_factor,
                   match   =>    [ @subs_match ],
 );
 
+sub GROUND { 1 }
+
 sub croak {
     require Carp;
     &Carp::croak;
@@ -43,12 +45,12 @@ Math::Factor - factorise integers and calculate matching multiplications.
  # data manipulation
  %factors = factor (\@numbers);
  %matches = match (\%factors);
- 
+
  # factors iteration
  while ($factor = each_factor(\%factors, $numbers[0])) {
      print "$factor\n";
  }
- 
+
  # matches iteration
  while (@match = each_match(\%matches, $numbers[0])) {
      print "$numbers[0] == $match[0] * $match[1]\n";
@@ -67,9 +69,21 @@ Factorises numbers.
  %factors = factor (\@numbers);
 
 Each number within @numbers will be entirely factorised and its factors will be
-saved within %factors, accessible by the number itself e.g  the factors of 9 may
+saved within %factors, accessible by the number itself e.g the factors of 9 may
 be accessed by @{$factors{9}}.
 
+Ranges may be evaluated by providing a two-dimensional array. 
+
+ @numbers = (
+     [ 9, '1-6' ],
+     [ 30107, '*' ],
+ );
+
+The first item (9) represents the number itself. The second item (1-6) represents
+the range that is being evaluated; ranges are indicated by a starting and ending
+value separated by a colon. Evaluating ranges for certain numbers may be entirely
+disabled by supplying *.
+
 =cut
 
 sub factor {
@@ -77,11 +91,26 @@ sub factor {
     croak q~usage: factor (\@numbers)~
       unless ref $data_ref eq 'ARRAY' && @$data_ref;
 
-    my %factor;
+    my (%factor, $number, $i, $limit);
     foreach (@$data_ref) {
-        for (my $i = 1; $i <= $_; $i++) {
-            if ($_ % $i == 0)  {
-                push @{$factor{$_}}, $i;
+        $i = 0; 
+        if (ref $_) { 
+	    $number = $$_[0];
+	    if ($$_[1] =~ /-/) { 
+    	        my @range = split /-/, $$_[1];
+		$i = $range[0];
+		if ($range[1] eq '*') { $limit = $number } 
+		else { $limit = $range[1] }
+	    }
+	    else { $limit = $number } 
+	}
+	else { 
+	    $number = $limit = $_;
+	}
+	$i = GROUND unless $i;
+        for (; $i <= $limit; $i++) {
+            if ($number % $i == 0)  {
+                push @{$factor{$number}}, $i;
             }
         }
     }
@@ -108,7 +137,7 @@ sub match {
     croak q~usage: match (\%factors)~
       unless ref $data_ref eq 'HASH' && %$data_ref;
 
-    my ($i, %match);
+    my (%match, $i);
     foreach (keys %$data_ref) {
         $i = 0;
         my @cmp = my @base = @{$$data_ref{$_}};
@@ -132,10 +161,14 @@ sub match {
 
 Returns each factor of a number in a scalar context.
 
- while ($factor = each_factor(\%factors, $numbers[0])) {
+ while ($factor = each_factor(\%factors, $number) {
      print "$factor\n";
  }
 
+If not all factors are being evaluated by each_factor(), 
+it is recommended to undef @{"Math::Factor::each_factor_$number"}  
+after usage of each_factor().
+
 =cut
 
 sub each_factor {
@@ -143,25 +176,29 @@ sub each_factor {
     croak q~usage: each_factor (\%factors, $number)~
       unless ref $data_ref eq 'HASH' && $number;
 
-    unless (${__PACKAGE__.'::each_factor'}) {
-        @{__PACKAGE__.'::each_factor'} = @{$$data_ref{$number}};
-        ${__PACKAGE__.'::each_factor'} = 1;
+    unless (${__PACKAGE__."::each_factor_$number"}) {
+        @{__PACKAGE__."::each_factor_$number"} = @{$$data_ref{$number}};
+        ${__PACKAGE__."::each_factor_$number"} = 1;
     }
 
-    if (@{__PACKAGE__.'::each_factor'}) {
-        return shift @{__PACKAGE__.'::each_factor'};
+    if (@{__PACKAGE__."::each_factor_$number"}) {
+        return shift @{__PACKAGE__."::each_factor_$number"};
     }
-    else { ${__PACKAGE__.'::each_factor'} = 0; return }
+    else { ${__PACKAGE__."::each_factor_$number"} = 0; return }
 }
 
 =head2 each_match
 
 Returns each match of a number in a array context.
 
- while (@match = each_match(\%matches, $numbers[0])) {
-     print "$numbers[0] == $match[0] * $match[1]\n";
+ while (@match = each_match(\%matches, $number) {
+     print "$number == $match[0] * $match[1]\n";
  }
 
+If not all matches are being evaluated by each_match(), 
+it is recommended to undef @{"Math::Factor::each_match_$number"}  
+after usage of each_match().
+
 =cut
 
 sub each_match {
@@ -169,18 +206,18 @@ sub each_match {
     croak q~usage: each_match (\%matches, $number)~
       unless ref $data_ref eq 'HASH' && $number;
 
-    unless (${__PACKAGE__.'::each_match'}) {
-        @{__PACKAGE__.'::each_match'} = @{$$data_ref{$number}};
-        ${__PACKAGE__.'::each_match'} = 1;
+    unless (${__PACKAGE__."::each_match_$number"}) {
+        @{__PACKAGE__."::each_match_$number"} = @{$$data_ref{$number}};
+        ${__PACKAGE__."::each_match_$number"} = 1;
     }
 
-    if (wantarray && @{__PACKAGE__.'::each_match'}) {
-        my @match = (${__PACKAGE__.'::each_match'}[0][0], 
-	  ${__PACKAGE__.'::each_match'}[0][1]);
-        splice (@{__PACKAGE__.'::each_match'}, 0, 1);
+    if (wantarray && @{__PACKAGE__."::each_match_$number"}) {
+        my @match = (${__PACKAGE__."::each_match_$number"}[0][0], 
+	  ${__PACKAGE__."::each_match_$number"}[0][1]);
+        splice (@{__PACKAGE__."::each_match_$number"}, 0, 1);
         return @match;
     }
-    else { ${__PACKAGE__.'::each_match'} = 0; return }
+    else { ${__PACKAGE__."::each_match_$number"} = 0; return }
 }
 
 1;
